% Perform GECP on a Cauchy-like matrix
%   M_(i,j) = d1_i * d2_j /(x_i - y_j)
% to high relative accuracy
% Inputs:
%   x = vector defining matrix M
%   y = vector defining matrix M
%   d1 = vector of row scale factors defining matrix M
%   d2 = vector of column scale factors defining matrix M
% Outputs:
%   pr = permutation array for rows
%   Pr = row permuation matrix = eye(pr,:)'
%   pc = permutation array for columns
%   Pc = columns permuation matrix = eye(:,pc)
%   L = unit lower triangular matrix
%   D = diagonal matrix
%   U = unit upper triangular matrix
%   (above matrices satisfy)
%       Pr*L*U*Pc'=M or Pr'*M*Pc=L*U
%   Mp = M(pr,pc) = Pr'*M*Pc
% Written by J. Demmel, 1997
%
% Appended by P. Koev, 2002 to handle x_i=y_j for V(x)=C*V(y)
%
% function [pr,pc,Pr,Pc,M,Mp,L,D,U,ierr] = CauchyGECP1(x,y,d1,d2)
function [pr,pc,Pr,Pc,M,Mp,L,D,U,ierr] = CauchyGECP1(x,y,d1,d2)
%
ierr = 0;
n = length(x);
xc = reshape(x,n,1);
yc = reshape(y,n,1);
pr = 1:n;
pc = 1:n;
on = ones(n,1);
%piv = 1./(xc*on' - on*yc');
%if (sum(sum(isnan(piv)+isinf(piv)))>0),
%   disp('NaNs and infinities in CauchyGECP(1)')
%   xc,yc,piv
%   ierr = 1;
%end
%piv = diag(d1)*piv*diag(d2);
%if (sum(sum(isnan(piv)+isinf(piv)))>0),
%   disp('NaNs and infinities in CauchyGECP(2)')
%   d1,d2,piv
%   ierr = 1;
%end
if (ierr==0),
fl = 2*n^2;

Z=zeros(n);
piv1=zeros(n);
d11=ones(n,1);
z1=zeros(n,1);
for i=1:n
   for j=1:n
      if xc(i)~=yc(j)
         d11(i)=d11(i)*(xc(i)-yc(j));
      else
         z1(i)=1;
      end
   end
end

for i=1:n
   for j=1:n
      if xc(i)~=yc(j) 
         piv1(i,j)=1/(xc(i)-yc(j));
      else
         piv1(i,j)=1;
         Z(i,j)=-1;
      end
      piv1(i,j)=d11(i)*piv1(i,j)*d2(j);
      Z(i,j)=Z(i,j)+z1(i);
      if Z(i,j)==0
         piv(i,j)=piv1(i,j);
      else
         piv(i,j)=0;
      end
   end
end

M = piv;

%diag(d1)*(1./(xc*on' - on*yc'))*diag(d2)
%
% Z
% piv

maxpiv = max(max(abs(piv)));
d = 0;
[r,c] = find(abs(piv) == maxpiv);
d(1) = piv(r(1),c(1));
if (r(1) ~= 1),
   tmp = pr(1);
   pr(1) = pr(r(1));
   pr(r(1)) = tmp;
   tmp = xc(1);
   xc(1) = xc(pr(1));
   xc(pr(1)) = tmp;
   tmp = piv(1,:);
   piv(1,:) = piv(pr(1),:);
   piv(pr(1),:) = tmp;
   tmp = piv1(1,:);
   piv1(1,:) = piv1(pr(1),:);
   piv1(pr(1),:) = tmp;
   tmp = Z(1,:);
   Z(1,:) = Z(pr(1),:);
   Z(pr(1),:) = tmp;
end
if (c(1) ~= 1),
   tmp = pc(1);
   pc(1) = pc(c(1));
   pc(c(1)) = tmp;
   tmp = yc(1);
   yc(1) = yc(pc(1));
   yc(pc(1)) = tmp;
   tmp = piv(:,1);
   piv(:,1) = piv(:,pc(1));
   piv(:,pc(1)) = tmp;
   tmp = piv1(:,1);
   piv1(:,1) = piv1(:,pc(1));
   piv1(:,pc(1)) = tmp;
   tmp = Z(:,1);
   Z(:,1) = Z(:,pc(1));
   Z(:,pc(1)) = tmp;
end
% piv,maxpiv,r,c,pr,pc
for i = 2:n,
   on = ones(n-i+1,1);
%   mul =          ( ( (xc(i:n) - xc(i-1)) * on' ) .* ...
%                  ( on * (yc(i:n) - yc(i-1))' ) ./ ...
%                  ( (xc(i:n) - yc(i-1)) * on' ) ./ ...
%                  ( on * (xc(i-1) - yc(i:n))' ) ) ;
               
   mul1=zeros(n-i+1);
   mulZ=zeros(n-i+1);            
   for j=i:n
      for k=i:n
         mul1(j-i+1,k-i+1)=(xc(j)-xc(i-1))*(yc(k)-yc(i-1));
         if xc(j)~=yc(i-1)
            mul1(j-i+1,k-i+1)=mul1(j-i+1,k-i+1)/(xc(j)-yc(i-1));
         else
            mulZ(j-i+1,k-i+1)=mulZ(j-i+1,k-i+1)-1;
         end
         if xc(i-1)~=yc(k)
            mul1(j-i+1,k-i+1)=mul1(j-i+1,k-i+1)/(xc(i-1)-yc(k));
         else
            mulZ(j-i+1,k-i+1)=mulZ(j-i+1,k-i+1)-1;
         end
      end
   end
   
%   norm(mul-mul1)   
%   if (sum(sum(isnan(mul) + isinf(mul)))>0), 
%      i, mul,
%      disp('pause 3'), pause,
%   end
%   oldpiv = piv(i:n,i:n);
   
   % piv(i:n,i:n) = -piv(i:n,i:n) .* mul ;
   piv1(i:n,i:n) = -piv1(i:n,i:n) .* mul1 ;
   Z(i:n,i:n)=Z(i:n,i:n)+mulZ;
   for j=i:n
      for k=i:n
         if Z(j,k)==0
            piv(j,k)=piv1(j,k);
         else
            piv(j,k)=0;
         end
         
      end
   end
   
 %  isn = sum(sum(isnan(piv)));
 %  if (isn>0),
 %     i, isn, piv, mul, oldpiv
 %     disp('pause 4'), pause,
 %  end
   fl = fl + 4*(n-i+1) + 4*(n-i+1)^2;
   maxpiv = max(max(abs(piv(i:n,i:n))));
   d(i) = maxpiv;
   [r,c] = find(abs(piv(i:n,i:n)) == maxpiv);
%   if (isn>0),
%      i, maxpiv, r, c
%      disp('pause 5'), pause,
%   end
   if (r(1) ~= 1),
      tmp = pr(i);
      pr(i) = pr(r(1)+i-1);
      pr(r(1)+i-1) = tmp;
      tmp = xc(i);
      xc(i) = xc(r(1)+i-1);
      xc(r(1)+i-1) = tmp;
      tmp = piv(i,:);
      piv(i,:) = piv(r(1)+i-1,:);
      piv(r(1)+i-1,:) = tmp;
      tmp = piv1(i,:);
      piv1(i,:) = piv1(r(1)+i-1,:);
      piv1(r(1)+i-1,:) = tmp;
      tmp = Z(i,:);
      Z(i,:) = Z(r(1)+i-1,:);
      Z(r(1)+i-1,:) = tmp;
   end
   if (c(1) ~= 1),
      tmp = pc(i);
      pc(i) = pc(c(1)+i-1);
      pc(c(1)+i-1) = tmp;
      tmp = yc(i);
      yc(i) = yc(c(1)+i-1);
      yc(c(1)+i-1) = tmp;
      tmp = piv(:,i);
      piv(:,i) = piv(:,c(1)+i-1);
      piv(:,c(1)+i-1) = tmp;
      tmp = piv1(:,i);
      piv1(:,i) = piv1(:,c(1)+i-1);
      piv1(:,c(1)+i-1) = tmp;
      tmp = Z(:,i);
      Z(:,i) = Z(:,c(1)+i-1);
      Z(:,c(1)+i-1) = tmp;
   end
%   piv,maxpiv,r,c,pr,pc
%   disp(['i=', int2str(i)]), pause
end
d(n) = piv(n,n);
d;
for i=1:n-1,
   piv(i+1:n,i) = piv(i+1:n,i)/piv(i,i);
   piv(i,i+1:n) = piv(i,i+1:n)/piv(i,i);
end
fl = fl + n*(n-1)/2;
Mp = M(pr,pc);
L = tril(piv,-1) + eye(n);
U = triu(piv,1)  + eye(n);
D = diag(diag(piv));
Pr = eye(n); Pr = Pr(pr,:)';
Pc = eye(n); Pc = Pc(:,pc);
else
  pr=1;pc=1;Pr=1;Pc=1;M=1;Mp=1;L=1;D=1;U=1;
end
