% Compute high relative accuracy SVD of a Cauchy-like matrix
%   M_(i,j) = d1_i * d2_j / ( x_i - y_j )
%
% Inputs:
%   x = array defining M as above
%   y = array defining M as above
%   d1 = array defining M as above
%   d2 = array defining M as above
%
% Outputs:
%   U = left singular vectors
%   S = diagonal matrix of singular values
%   V = right singular vectors
%        such that M = U*S*V' to high relative accuracy
%   relerrbnd = relative error bound from Thm 3.1 in DGESVD paper
%        (relative error bound on singular values)
%        (divided by relative gap to get error bound on singular vectors)
%   Jnum = number of Jacobi sweeps
%
% Written by J. Demmel, 1997, demmel@cs.berkeley.edu
%
% function [U,S,V,relerrbnd,Jnum] = CauchySVD(x,y,d1,d2)
function [U,S,V,relerrbnd,Jnum] = CauchySVD1(x,y,d1,d2)
%
% Perform GECP to high relative accuracy
[pr,pc,Pr,Pc,M,Mp,L,D,U] = CauchyGECP1(x,y,d1,d2);
%
% Compute Rank Revealing Decomposition M = X*D*Y^T
X = Pr*L;
Y = (U*Pc')';
%
% Compute SVD using algorithm from DGESVD
[U,S,V,relerrbnd,Jnum]=accsvd1(X,D,Y);
